@@ -39,11 +39,13 @@
 #include <cregistry/registry.h>
 #include <cregistry/portgroup.h>
 #include <cregistry/entry.h>
+#include <cregistry/snapshot.h>
 #include <cregistry/file.h>
 
 #include "entry.h"
 #include "entryobj.h"
 #include "file.h"
+#include "snapshot.h"
 #include "portgroup.h"
 #include "registry.h"
 #include "util.h"
@@ -140,10 +142,18 @@ int registry_tcl_detach(Tcl_Interp* interp, reg_registry* reg,
 static void delete_reg(ClientData reg, Tcl_Interp* interp) {
     reg_error error;
     if (((reg_registry*)reg)->status & reg_attached) {
+        if (reg_optimize((reg_registry*)reg, &error) == 0) {
+            fprintf(stderr, "%s\n", error.description);
+            reg_error_destruct(&error);
+        }
         if (Tcl_GetAssocData(interp, "registry::needs_vacuum", NULL) != NULL) {
             reg_vacuum(Tcl_GetAssocData(interp, "registry::db_path", NULL));
             Tcl_DeleteAssocData(interp, "registry::needs_vacuum");
         }
+        if (reg_checkpoint((reg_registry*)reg, &error) == 0) {
+            fprintf(stderr, "%s\n", error.description);
+            reg_error_destruct(&error);
+        }
         if (!registry_tcl_detach(interp, (reg_registry*)reg, &error)) {
             fprintf(stderr, "%s", error.description);
             reg_error_destruct(&error);
@@ -234,6 +244,10 @@ static int registry_close(ClientData clientData UNUSED, Tcl_Interp* interp,
             return TCL_ERROR;
         } else {
             reg_error error;
+            if (reg_optimize(reg, &error) == 0) {
+                fprintf(stderr, "%s\n", error.description);
+                reg_error_destruct(&error);
+            }
             if (Tcl_GetAssocData(interp, "registry::needs_vacuum", NULL) != NULL) {
                 reg_vacuum(Tcl_GetAssocData(interp, "registry::db_path", NULL));
                 Tcl_DeleteAssocData(interp, "registry::needs_vacuum");
@@ -241,6 +255,7 @@ static int registry_close(ClientData clientData UNUSED, Tcl_Interp* interp,
             /* Not really anything we can do if this fails. */
             if (reg_checkpoint(reg, &error) == 0) {
                 fprintf(stderr, "%s\n", error.description);
+                reg_error_destruct(&error);
             }
             if (registry_tcl_detach(interp, reg, &error)) {
                 return TCL_OK;
@@ -388,6 +403,17 @@ int metadata_cmd(ClientData clientData UNUSED, Tcl_Interp* interp, int objc,
     return TCL_ERROR;
 }
 
+/* Allow setting the needs_vacuum flag from scripts. */
+int set_needs_vacuum_cmd(ClientData clientData UNUSED, Tcl_Interp* interp, int objc,
+        Tcl_Obj* const objv[]) {
+    if (objc != 1) {
+        Tcl_WrongNumArgs(interp, 1, objv, "cmd");
+        return TCL_ERROR;
+    }
+    Tcl_SetAssocData(interp, "registry::needs_vacuum", NULL, (ClientData)1);
+    return TCL_OK;
+}
+
 /**
  * Initializer for the registry lib.
  *
@@ -403,9 +429,11 @@ int Registry_Init(Tcl_Interp* interp) {
     Tcl_CreateObjCommand(interp, "registry::read", registry_read, NULL, NULL);
     Tcl_CreateObjCommand(interp, "registry::write", registry_write, NULL, NULL);
     Tcl_CreateObjCommand(interp, "registry::entry", entry_cmd, NULL, NULL);
+    Tcl_CreateObjCommand(interp, "registry::snapshot", snapshot_cmd, NULL, NULL);
     Tcl_CreateObjCommand(interp, "registry::file", file_cmd, NULL, NULL);
     Tcl_CreateObjCommand(interp, "registry::portgroup", portgroup_cmd, NULL, NULL);
     Tcl_CreateObjCommand(interp, "registry::metadata", metadata_cmd, NULL, NULL);
+    Tcl_CreateObjCommand(interp, "registry::set_needs_vacuum", set_needs_vacuum_cmd, NULL, NULL);
     if (Tcl_PkgProvide(interp, "registry2", "2.0") != TCL_OK) {
         return TCL_ERROR;
     }
